Module: Test::Unit::CoreAssertions
- Defined in:
- lib/core_assertions.rb
Defined Under Namespace
Classes: AllFailures
Constant Summary collapse
- FailDesc =
proc do |status, = "", out = ""| now = Time.now proc do EnvUtil.failure_description(status, now, , out) end end
- ABORT_SIGNALS =
Signal.list.values_at(*%w"ILL ABRT BUS SEGV TERM")
- TEST_DIR =
File.join(__dir__, "test/unit")
Class Method Summary collapse
- .glibc?(*ver) ⇒ Boolean
- .linux?(*ver) ⇒ Boolean
- .macos?(*ver) ⇒ Boolean
- .mingw? ⇒ Boolean
-
.mswin? ⇒ Boolean
Platform predicates.
- .sanitizers ⇒ Object
- .version_compare(expected, actual) ⇒ Object
- .version_match?(expected, actual) ⇒ Boolean
- .windows? ⇒ Boolean
Instance Method Summary collapse
-
#assert(test, *msgs) ⇒ Object
:call-seq: assert(test, [failure_message]).
- #assert_all?(obj, m = nil, &blk) ⇒ Boolean
- #assert_all_assertions(msg = nil) ⇒ Object (also: #all_assertions)
- #assert_all_assertions_foreach(msg = nil, *keys, &block) ⇒ Object (also: #all_assertions_foreach)
- #assert_deprecated_warn(mesg = /deprecated/, &block) ⇒ Object
- #assert_deprecated_warning(mesg = /deprecated/, &block) ⇒ Object
- #assert_file ⇒ Object
- #assert_in_out_err(args, test_stdin = "", test_stdout = [], test_stderr = [], message = nil, success: nil, failed: nil, gems: false, **opt) ⇒ Object
-
#assert_join_threads(threads, message = nil) ⇒ Object
threads should respond to shift method.
-
#assert_linear_performance(seq, rehearsal: nil, pre: ->(n) {n}) ⇒ Object
Expect
seqto respond tofirstandeachmethods, e.g., Array, Range, Enumerator::ArithmeticSequence and other Enumerable-s, and each elements should be size factors. - #assert_no_memory_leak(args, prepare, code, message = nil, limit: 2.0, rss: false, **opt) ⇒ Object
- #assert_normal_exit(testsrc, message = '', child_env: nil, **opt) ⇒ Object
-
#assert_not_respond_to(obj, meth, *priv, msg = nil) ⇒ Object
:call-seq: assert_not_respond_to( object, method, failure_message = nil ).
-
#assert_nothing_raised(*args) ⇒ Object
:call-seq: assert_nothing_raised( *args, &block ).
-
#assert_pattern_list(pattern_list, actual, message = nil) ⇒ Object
pattern_list is an array which contains regexp, string and :*.
-
#assert_ractor(src, args: [], require: nil, require_relative: nil, file: nil, line: nil, ignore_stderr: nil, **opt) ⇒ Object
Run Ractor-related test without influencing the main test suite.
-
#assert_raise(*exp, &b) ⇒ Object
:call-seq: assert_raise( *args, &block ).
-
#assert_raise_kind_of(*exp, &b) ⇒ Object
:call-seq: assert_raise_kind_of(*args, &block).
-
#assert_raise_with_message(exception, expected, msg = nil, &block) ⇒ Object
:call-seq: assert_raise_with_message(exception, expected, msg = nil, &block).
-
#assert_respond_to(obj, meth, *priv, msg = nil) ⇒ Object
:call-seq: assert_respond_to( object, method, failure_message = nil ).
- #assert_ruby_status(args, test_stdin = "", message = nil, **opt) ⇒ Object
- #assert_separately(args, file = nil, line = nil, src, ignore_stderr: nil, **opt) ⇒ Object
-
#assert_throw(tag, msg = nil) ⇒ Object
:call-seq: assert_throw( tag, failure_message = nil, &block ).
- #assert_valid_syntax(code, *args, **opt) ⇒ Object
- #assert_warn(*args) ⇒ Object
- #assert_warning(pat, msg = nil) ⇒ Object
- #diff(exp, act) ⇒ Object
- #mu_pp(obj) ⇒ Object
- #new_test_token ⇒ Object
- #prepare_syntax_check(code, fname = nil, mesg = nil, verbose: nil) ⇒ Object
- #separated_runner(token, out = nil) ⇒ Object
- #syntax_check(code, fname, line) ⇒ Object
Class Method Details
.glibc?(*ver) ⇒ Boolean
1010 1011 1012 1013 1014 1015 1016 1017 1018 1019 1020 |
# File 'lib/core_assertions.rb', line 1010 def self.glibc?(*ver) unless defined?(@glibc) libc = `/usr/bin/ldd /bin/sh`[/^\s*libc.*=> *\K\S*/] if libc and /version (\d+)\.(\d+)\.$/ =~ IO.popen([libc], &:read)[] @glibc = [$1.to_i, $2.to_i] else @glibc = false end end version_match? ver, @glibc end |
.linux?(*ver) ⇒ Boolean
1000 1001 1002 1003 1004 1005 |
# File 'lib/core_assertions.rb', line 1000 def self.linux?(*ver) unless defined?(@linux) @linux = RUBY_PLATFORM.include?('linux') && `uname -r`.scan(/\d+/).map(&:to_i) end version_match? ver, @linux end |
.macos?(*ver) ⇒ Boolean
1025 1026 1027 1028 1029 1030 |
# File 'lib/core_assertions.rb', line 1025 def self.macos?(*ver) unless defined?(@macos) @macos = RUBY_PLATFORM.include?('darwin') && `sw_vers -productVersion`.scan(/\d+/).map(&:to_i) end version_match? ver, @macos end |
.mingw? ⇒ Boolean
968 969 970 |
# File 'lib/core_assertions.rb', line 968 def self.mingw? defined?(@mingw) ? @mingw : @mingw = RUBY_PLATFORM.include?('mingw') end |
.mswin? ⇒ Boolean
Platform predicates
961 962 963 |
# File 'lib/core_assertions.rb', line 961 def self.mswin? defined?(@mswin) ? @mswin : @mswin = RUBY_PLATFORM.include?('mswin') end |
.sanitizers ⇒ Object
.version_compare(expected, actual) ⇒ Object
979 980 981 982 |
# File 'lib/core_assertions.rb', line 979 def self.version_compare(expected, actual) expected.zip(actual).each {|e, a| z = (e <=> a); return z if z.nonzero?} 0 end |
.version_match?(expected, actual) ⇒ Boolean
984 985 986 987 988 989 990 991 992 993 994 995 996 997 998 |
# File 'lib/core_assertions.rb', line 984 def self.version_match?(expected, actual) if !actual false elsif expected.empty? true elsif expected.size == 1 and Range === (range = expected.first) b, e = range.begin, range.end return false if b and (c = version_compare(Array(b), actual)) > 0 return false if e and (c = version_compare(Array(e), actual)) < 0 return false if e and range.exclude_end? and c == 0 true else version_compare(expected, actual).zero? end end |
.windows? ⇒ Boolean
975 976 977 |
# File 'lib/core_assertions.rb', line 975 module_function def windows? mswin? or mingw? end |
Instance Method Details
#assert(test, *msgs) ⇒ Object
:call-seq:
assert(test, [])
Tests if test is true.
msg may be a String or a Proc. If msg is a String, it will be used
as the failure message. Otherwise, the result of calling msg will be
used as the message if the assertion fails.
If no msg is given, a default message will be used.
assert(false, "This was expected to be true")
623 624 625 626 627 628 629 630 631 632 633 |
# File 'lib/core_assertions.rb', line 623 def assert(test, *msgs) case msg = msgs.first when String, Proc when nil msgs.shift else bt = caller.reject { |s| s.start_with?(TEST_DIR) } raise ArgumentError, "assertion message must be String or Proc, but #{msg.class} was given.", bt end unless msgs.empty? super end |
#assert_all?(obj, m = nil, &blk) ⇒ Boolean
850 851 852 853 854 855 856 857 858 |
# File 'lib/core_assertions.rb', line 850 def assert_all?(obj, m = nil, &blk) failed = [] obj.each do |*a, &b| unless blk.call(*a, &b) failed << (a.size > 1 ? a : a[0]) end end assert(failed.empty?, (m) {failed.pretty_inspect}) end |
#assert_all_assertions(msg = nil) ⇒ Object Also known as: all_assertions
860 861 862 863 864 865 |
# File 'lib/core_assertions.rb', line 860 def assert_all_assertions(msg = nil) all = AllFailures.new yield all ensure assert(all.pass?, (msg) {all..chomp(".")}) end |
#assert_all_assertions_foreach(msg = nil, *keys, &block) ⇒ Object Also known as: all_assertions_foreach
868 869 870 871 872 873 |
# File 'lib/core_assertions.rb', line 868 def assert_all_assertions_foreach(msg = nil, *keys, &block) all = AllFailures.new all.foreach(*keys, &block) ensure assert(all.pass?, (msg) {all..chomp(".")}) end |
#assert_deprecated_warn(mesg = /deprecated/, &block) ⇒ Object
748 749 750 751 752 |
# File 'lib/core_assertions.rb', line 748 def assert_deprecated_warn(mesg = /deprecated/, &block) assert_warn(mesg) do EnvUtil.deprecation_warning(&block) end end |
#assert_deprecated_warning(mesg = /deprecated/, &block) ⇒ Object
742 743 744 745 746 |
# File 'lib/core_assertions.rb', line 742 def assert_deprecated_warning(mesg = /deprecated/, &block) assert_warning(mesg) do EnvUtil.deprecation_warning(&block) end end |
#assert_file ⇒ Object
97 98 99 |
# File 'lib/core_assertions.rb', line 97 def assert_file AssertFile end |
#assert_in_out_err(args, test_stdin = "", test_stdout = [], test_stderr = [], message = nil, success: nil, failed: nil, gems: false, **opt) ⇒ Object
108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 |
# File 'lib/core_assertions.rb', line 108 def assert_in_out_err(args, test_stdin = "", test_stdout = [], test_stderr = [], = nil, success: nil, failed: nil, gems: false, **opt) args = Array(args).dup unless gems.nil? args.insert((Hash === args[0] ? 1 : 0), "--#{gems ? 'enable' : 'disable'}=gems") end stdout, stderr, status = EnvUtil.invoke_ruby(args, test_stdin, true, true, **opt) desc = failed[status, , stderr] if failed desc ||= FailDesc[status, , stderr] if block_given? raise "test_stdout ignored, use block only or without block" if test_stdout != [] raise "test_stderr ignored, use block only or without block" if test_stderr != [] yield(stdout.lines.map {|l| l.chomp }, stderr.lines.map {|l| l.chomp }, status) else all_assertions(desc) do |a| [["stdout", test_stdout, stdout], ["stderr", test_stderr, stderr]].each do |key, exp, act| a.for(key) do if exp.is_a?(Regexp) assert_match(exp, act) elsif exp.all? {|e| String === e} assert_equal(exp, act.lines.map {|l| l.chomp }) else assert_pattern_list(exp, act) end end end unless success.nil? a.for("success?") do if success assert_predicate(status, :success?) else assert_not_predicate(status, :success?) end end end end status end end |
#assert_join_threads(threads, message = nil) ⇒ Object
threads should respond to shift method. Array can be used.
820 821 822 823 824 825 826 827 828 829 830 831 832 833 834 835 836 837 838 839 840 841 842 843 844 845 846 847 848 |
# File 'lib/core_assertions.rb', line 820 def assert_join_threads(threads, = nil) errs = [] values = [] while th = threads.shift begin values << th.value rescue Exception errs << [th, $!] th = nil end end values ensure if th&.alive? th.raise(Timeout::Error.new) th.join rescue errs << [th, $!] end if !errs.empty? msg = "exceptions on #{errs.length} threads:\n" + errs.map {|t, err| "#{t.inspect}:\n" + (err.respond_to?(:full_message) ? err.(highlight: false, order: :top) : err.) }.join("\n---\n") if msg = "#{}\n#{msg}" end raise Test::Unit::AssertionFailedError, msg end end |
#assert_linear_performance(seq, rehearsal: nil, pre: ->(n) {n}) ⇒ Object
Expect seq to respond to first and each methods, e.g.,
Array, Range, Enumerator::ArithmeticSequence and other
Enumerable-s, and each elements should be size factors.
:yield: each elements of seq.
901 902 903 904 905 906 907 908 909 910 911 912 913 914 915 916 917 918 919 920 921 922 923 924 925 926 927 928 929 930 931 932 933 934 935 936 |
# File 'lib/core_assertions.rb', line 901 def assert_linear_performance(seq, rehearsal: nil, pre: ->(n) {n}) pend "No PERFORMANCE_CLOCK found" unless defined?(PERFORMANCE_CLOCK) # Timeout testing generally doesn't work when RJIT compilation happens. rjit_enabled = defined?(RubyVM::RJIT) && RubyVM::RJIT.enabled? measure = proc do |arg, | st = Process.clock_gettime(PERFORMANCE_CLOCK) yield(*arg) t = (Process.clock_gettime(PERFORMANCE_CLOCK) - st) assert_operator 0, :<=, t, unless rjit_enabled t end first = seq.first *arg = pre.call(first) raw_times = (0..(rehearsal || (2 * first))).map do measure[arg, "rehearsal"].nonzero? end times = raw_times.compact raise "all measurements are zero: #{raw_times.inspect}" if times.empty? tmin, tmax = times.minmax # safe_factor * tmax * rehearsal_time_variance_factor(equals to 1 when variance is small) tbase = 10 * tmax * [(tmax / tmin) ** 2 / 4, 1].max info = "(tmin: #{tmin}, tmax: #{tmax}, tbase: #{tbase})" seq.each do |i| next if i == first t = tbase * i.fdiv(first) *arg = pre.call(i) = "[#{i}]: in #{t}s #{info}" Timeout.timeout(t, Timeout::Error, ) do measure[arg, ] end end end |
#assert_no_memory_leak(args, prepare, code, message = nil, limit: 2.0, rss: false, **opt) ⇒ Object
167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 |
# File 'lib/core_assertions.rb', line 167 def assert_no_memory_leak(args, prepare, code, =nil, limit: 2.0, rss: false, **opt) # TODO: consider choosing some appropriate limit for RJIT and stop skipping this once it does not randomly fail pend 'assert_no_memory_leak may consider RJIT memory usage as leak' if defined?(RubyVM::RJIT) && RubyVM::RJIT.enabled? # For previous versions which implemented MJIT pend 'assert_no_memory_leak may consider MJIT memory usage as leak' if defined?(RubyVM::MJIT) && RubyVM::MJIT.enabled? # ASAN has the same problem - its shadow memory greatly increases memory usage # (plus asan has better ways to detect memory leaks than this assertion) pend 'assert_no_memory_leak may consider ASAN memory usage as leak' if sanitizers&.asan_enabled? require_relative 'memory_status' raise Test::Unit::PendedError, "unsupported platform" unless defined?(Memory::Status) token_dump, token_re = new_test_token envs = args.shift if Array === args and Hash === args.first args = [ "--disable=gems", "-r", File.("../memory_status", __FILE__), *args, "-v", "-", ] if defined? Memory::NO_MEMORY_LEAK_ENVS then envs ||= {} newenvs = envs.merge(Memory::NO_MEMORY_LEAK_ENVS) { |_, _, _| break } envs = newenvs if newenvs end args.unshift(envs) if envs cmd = [ 'END {STDERR.puts '"#{token_dump}"'"FINAL=#{Memory::Status.new}"}', prepare, 'STDERR.puts('"#{token_dump}"'"START=#{$initial_status = Memory::Status.new}")', '$initial_size = $initial_status.size', code, 'GC.start', ].join("\n") _, err, status = EnvUtil.invoke_ruby(args, cmd, true, true, **opt) before = err.sub!(/^#{token_re}START=(\{.*\})\n/, '') && Memory::Status.parse($1) after = err.sub!(/^#{token_re}FINAL=(\{.*\})\n/, '') && Memory::Status.parse($1) assert(status.success?, FailDesc[status, , err]) ([:size, (rss && :rss)] & after.members).each do |n| b = before[n] a = after[n] next unless a > 0 and b > 0 assert_operator(a.fdiv(b), :<, limit, () {"#{n}: #{b} => #{a}"}) end rescue LoadError pend end |
#assert_normal_exit(testsrc, message = '', child_env: nil, **opt) ⇒ Object
283 284 285 286 287 288 289 290 291 292 |
# File 'lib/core_assertions.rb', line 283 def assert_normal_exit(testsrc, = '', child_env: nil, **opt) assert_valid_syntax(testsrc, caller_locations(1, 1)[0]) if child_env child_env = [child_env] else child_env = [] end out, _, status = EnvUtil.invoke_ruby(child_env + %W'-W0', testsrc, true, :merge_to_stdout, **opt) assert !status.signaled?, FailDesc[status, , out] end |
#assert_not_respond_to(obj, meth, *priv, msg = nil) ⇒ Object
:call-seq:
assert_not_respond_to( object, method, = nil )
Tests if the given Object does not respond to method.
An optional failure message may be provided as the final argument.
assert_not_respond_to("hello", :reverse) #Fails
assert_not_respond_to("hello", :does_not_exist) #Succeeds
667 668 669 670 671 672 673 674 675 676 677 678 679 |
# File 'lib/core_assertions.rb', line 667 def assert_not_respond_to(obj, (meth, *priv), msg = nil) unless priv.empty? msg = (msg) { "Expected #{mu_pp(obj)} (#{obj.class}) to not respond to ##{meth}#{" privately" if priv[0]}" } return assert !obj.respond_to?(meth, *priv), msg end #get rid of overcounting if caller_locations(1, 1)[0].path.start_with?(TEST_DIR) return unless obj.respond_to?(meth) end refute_respond_to(obj, meth, msg) end |
#assert_nothing_raised(*args) ⇒ Object
:call-seq:
assert_nothing_raised( *args, &block )
If any exceptions are given as arguments, the assertion will fail if one of those exceptions are raised. Otherwise, the test fails if any exceptions are raised.
The final argument may be a failure message.
assert_nothing_raised RuntimeError do
raise Exception #Assertion passes, Exception is not a RuntimeError
end
assert_nothing_raised do
raise Exception #Assertion fails
end
231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 |
# File 'lib/core_assertions.rb', line 231 def assert_nothing_raised(*args) self._assertions += 1 if Module === args.last msg = nil else msg = args.pop end begin yield rescue Test::Unit::PendedError, *(Test::Unit::AssertionFailedError if args.empty?) raise rescue *(args.empty? ? Exception : args) => e msg = (msg) { "Exception raised:\n<#{mu_pp(e)}>\n""Backtrace:\n" << Test.filter_backtrace(e.backtrace).map{|frame| " #{frame}"}.join("\n") } raise Test::Unit::AssertionFailedError, msg.call, e.backtrace end end |
#assert_pattern_list(pattern_list, actual, message = nil) ⇒ Object
pattern_list is an array which contains regexp, string and :. : means any sequence.
pattern_list is anchored. Use [:, regexp/string, :] for non-anchored match.
686 687 688 689 690 691 692 693 694 695 696 697 698 699 700 701 702 703 704 705 706 707 708 709 710 711 712 713 714 715 716 717 718 719 720 721 722 723 724 |
# File 'lib/core_assertions.rb', line 686 def assert_pattern_list(pattern_list, actual, =nil) rest = actual anchored = true pattern_list.each_with_index {|pattern, i| if pattern == :* anchored = false else if anchored match = rest.rindex(pattern, 0) else match = rest.index(pattern) end if match post_match = $~ ? $~.post_match : rest[match+pattern.size..-1] else msg = (msg) { expect_msg = "Expected #{mu_pp pattern}\n" if /\n[^\n]/ =~ rest actual_mesg = +"to match\n" rest.scan(/.*\n+/) { actual_mesg << ' ' << $&.inspect << "+\n" } actual_mesg.sub!(/\+\n\z/, '') else actual_mesg = "to match " + mu_pp(rest) end actual_mesg << "\nafter #{i} patterns with #{actual.length - rest.length} characters" expect_msg + actual_mesg } assert false, msg end rest = post_match anchored = true end } if anchored assert_equal("", rest) end end |
#assert_ractor(src, args: [], require: nil, require_relative: nil, file: nil, line: nil, ignore_stderr: nil, **opt) ⇒ Object
Run Ractor-related test without influencing the main test suite
420 421 422 423 424 425 426 427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 443 444 445 446 447 448 |
# File 'lib/core_assertions.rb', line 420 def assert_ractor(src, args: [], require: nil, require_relative: nil, file: nil, line: nil, ignore_stderr: nil, **opt) omit unless defined?(Ractor) # https://bugs.ruby-lang.org/issues/21262 shim_value = "class Ractor; alias value take; end" unless Ractor.method_defined?(:value) shim_join = "class Ractor; alias join take; end" unless Ractor.method_defined?(:join) if require require = [require] unless require.is_a?(Array) require = require.map {|r| "require #{r.inspect}"}.join("\n") end if require_relative dir = File.dirname(caller_locations[0,1][0].absolute_path) full_path = File.(require_relative, dir) require = "#{require}; require #{full_path.inspect}" end assert_separately(args, file, line, <<~RUBY, ignore_stderr: ignore_stderr, **opt) #{shim_value} #{shim_join} #{require} previous_verbose = $VERBOSE $VERBOSE = nil Ractor.new {} # trigger initial warning $VERBOSE = previous_verbose #{src} RUBY end |
#assert_raise(*exp, &b) ⇒ Object
:call-seq:
assert_raise( *args, &block )
Tests if the given block raises an exception. Acceptable exception types may be given as optional arguments. If the last argument is a String, it will be used as the error message.
assert_raise do #Fails, no Exceptions are raised
end
assert_raise NameError do
puts x #Raises NameError, so assertion succeeds
end
492 493 494 495 496 497 498 499 500 501 502 503 504 505 506 507 508 509 510 511 512 513 514 515 516 517 518 519 520 521 522 523 524 |
# File 'lib/core_assertions.rb', line 492 def assert_raise(*exp, &b) case exp.last when String, Proc msg = exp.pop end begin yield rescue Test::Unit::PendedError => e return e if exp.include? Test::Unit::PendedError raise e rescue Exception => e expected = exp.any? { |ex| if ex.instance_of? Module then e.kind_of? ex else e.instance_of? ex end } assert expected, proc { flunk((msg) {"#{mu_pp(exp)} exception expected, not #{mu_pp(e)}"}) } return e ensure unless e exp = exp.first if exp.size == 1 flunk((msg) {"#{mu_pp(exp)} expected but nothing was raised"}) end end end |
#assert_raise_kind_of(*exp, &b) ⇒ Object
:call-seq:
assert_raise_kind_of(*args, &block)
Tests if the given block raises one of the given exceptions or sub exceptions of the given exceptions. If the last argument is a String, it will be used as the error message.
assert_raise do #Fails, no Exceptions are raised
end
assert_raise SystemCallErr do
Dir.chdir(__FILE__) #Raises Errno::ENOTDIR, so assertion succeeds
end
585 586 587 588 589 590 591 592 593 594 595 596 597 598 599 600 601 602 603 604 605 606 607 |
# File 'lib/core_assertions.rb', line 585 def assert_raise_kind_of(*exp, &b) case exp.last when String, Proc msg = exp.pop end begin yield rescue Test::Unit::PendedError => e raise e unless exp.include? Test::Unit::PendedError rescue *exp => e pass rescue Exception => e flunk((msg) {"#{mu_pp(exp)} family exception expected, not #{mu_pp(e)}"}) ensure unless e exp = exp.first if exp.size == 1 flunk((msg) {"#{mu_pp(exp)} family expected but nothing was raised"}) end end e end |
#assert_raise_with_message(exception, expected, msg = nil, &block) ⇒ Object
:call-seq:
(exception, expected, msg = nil, &block)
Tests if the given block raises an exception with the expected message.
(RuntimeError, "foo") do
nil #Fails, no Exceptions are raised
end
(RuntimeError, "foo") do
raise ArgumentError, "foo" #Fails, different Exception is raised
end
(RuntimeError, "foo") do
raise "bar" #Fails, RuntimeError is raised but the message differs
end
(RuntimeError, "foo") do
raise "foo" #Raises RuntimeError with the message, so assertion succeeds
end
547 548 549 550 551 552 553 554 555 556 557 558 559 560 561 562 563 564 565 566 567 568 569 570 |
# File 'lib/core_assertions.rb', line 547 def (exception, expected, msg = nil, &block) case expected when String assert = :assert_equal else assert_respond_to(expected, :===) assert = :assert_match end ex = assert_raise(exception, msg || proc {"Exception(#{exception}) with message matches to #{expected.inspect}"}) do yield end m = ex. msg = (msg, "") {"Expected Exception(#{exception}) was raised, but the message doesn't match"} if assert == :assert_equal assert_equal(expected, m, msg) else msg = (msg) { "Expected #{mu_pp expected} to match #{mu_pp m}" } assert expected =~ m, msg block.binding.eval("proc{|_|$~=_}").call($~) end ex end |
#assert_respond_to(obj, meth, *priv, msg = nil) ⇒ Object
:call-seq:
assert_respond_to( object, method, = nil )
Tests if the given Object responds to method.
An optional failure message may be provided as the final argument.
assert_respond_to("hello", :reverse) #Succeeds
assert_respond_to("hello", :does_not_exist) #Fails
644 645 646 647 648 649 650 651 652 653 654 655 656 |
# File 'lib/core_assertions.rb', line 644 def assert_respond_to(obj, (meth, *priv), msg = nil) unless priv.empty? msg = (msg) { "Expected #{mu_pp(obj)} (#{obj.class}) to respond to ##{meth}#{" privately" if priv[0]}" } return assert obj.respond_to?(meth, *priv), msg end #get rid of overcounting if caller_locations(1, 1)[0].path.start_with?(TEST_DIR) return if obj.respond_to?(meth) end super(obj, meth, msg) end |
#assert_ruby_status(args, test_stdin = "", message = nil, **opt) ⇒ Object
294 295 296 297 298 299 300 |
# File 'lib/core_assertions.rb', line 294 def assert_ruby_status(args, test_stdin="", =nil, **opt) out, _, status = EnvUtil.invoke_ruby(args, test_stdin, true, :merge_to_stdout, **opt) desc = FailDesc[status, , out] assert(!status.signaled?, desc) ||= "ruby exit status is not success:" assert(status.success?, desc) end |
#assert_separately(args, file = nil, line = nil, src, ignore_stderr: nil, **opt) ⇒ Object
342 343 344 345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 379 380 381 382 383 384 385 386 387 388 389 390 391 392 393 394 395 396 397 398 399 400 401 402 403 404 405 406 407 408 409 410 411 412 413 414 415 416 417 |
# File 'lib/core_assertions.rb', line 342 def assert_separately(args, file = nil, line = nil, src, ignore_stderr: nil, **opt) unless file and line loc, = caller_locations(1,1) file ||= loc.path line ||= loc.lineno end capture_stdout = true unless /mswin|mingw/ =~ RbConfig::CONFIG['host_os'] capture_stdout = false opt[:out] = Test::Unit::Runner.output if defined?(Test::Unit::Runner) res_p, res_c = IO.pipe opt[:ios] = [res_c] end token_dump, token_re = new_test_token src = <<eom # -*- coding: #{line += __LINE__; src.encoding}; -*- BEGIN { require "test/unit";include Test::Unit::Assertions;require #{__FILE__.dump};include Test::Unit::CoreAssertions separated_runner #{token_dump}, #{res_c&.fileno || 'nil'} } #{line -= __LINE__; src} eom args = args.dup args.insert((Hash === args.first ? 1 : 0), "-w", "--disable=gems", *$:.map {|l| "-I#{l}"}) args << "--debug" if RUBY_ENGINE == 'jruby' # warning: tracing (e.g. set_trace_func) will not capture all events without --debug flag # power_assert 3 requires ruby 3.1 or later args << "-W:no-experimental" if ("2.7."..."3.1.").cover?(RUBY_VERSION) stdout, stderr, status = EnvUtil.invoke_ruby(args, src, capture_stdout, true, **opt) if sanitizers&.lsan_enabled? # LSAN may output messages like the following line into stderr. We should ignore it. # ==276855==Running thread 276851 was not suspended. False leaks are possible. # See https://github.com/google/sanitizers/issues/1479 stderr.gsub!(/==\d+==Running thread \d+ was not suspended\. False leaks are possible\.\n/, "") end ensure if res_c res_c.close res = res_p.read res_p.close else res = stdout end raise if $! abort = status.coredump? || (status.signaled? && ABORT_SIGNALS.include?(status.termsig)) marshal_error = nil assert(!abort, FailDesc[status, nil, stderr]) res.scan(/^<error id="#{token_re}" assertions=(\d+)>\n(.*?)\n(?=<\/error id="#{token_re}">$)/m) do self._assertions += $1.to_i begin res = Marshal.load($2.unpack1("m")) or next rescue => marshal_error ignore_stderr = nil res = nil else next if SystemExit === res if bt = res.backtrace bt.each do |l| l.sub!(/\A-:(\d+)/){"#{file}:#{line + $1.to_i}"} end bt.concat(caller) else res.set_backtrace(caller) end raise res end end # really did it succeed? unless ignore_stderr # the body of assert_separately must not output anything to detect error assert(stderr.empty?, FailDesc[status, "assert_separately failed with error message", stderr]) end assert(status.success?, FailDesc[status, "assert_separately failed", stderr]) raise marshal_error if marshal_error end |
#assert_throw(tag, msg = nil) ⇒ Object
:call-seq:
assert_throw( tag, = nil, &block )
Fails unless the given block throws tag, returns the caught
value otherwise.
An optional failure message may be provided as the final argument.
tag = Object.new
assert_throw(tag, "#{tag} was not thrown!") do
throw tag
end
462 463 464 465 466 467 468 469 470 471 472 473 474 475 476 477 |
# File 'lib/core_assertions.rb', line 462 def assert_throw(tag, msg = nil) ret = catch(tag) do begin yield(tag) rescue UncaughtThrowError => e thrown = e.tag end msg = (msg) { "Expected #{mu_pp(tag)} to have been thrown"\ "#{%Q[, not #{thrown}] if thrown}" } assert(false, msg) end assert(true) ret end |
#assert_valid_syntax(code, *args, **opt) ⇒ Object
274 275 276 277 278 279 280 281 |
# File 'lib/core_assertions.rb', line 274 def assert_valid_syntax(code, *args, **opt) prepare_syntax_check(code, *args, **opt) do |src, fname, line, mesg| yield if defined?(yield) assert_nothing_raised(SyntaxError, mesg) do assert_equal(:ok, syntax_check(src, fname, line), mesg) end end end |
#assert_warn(*args) ⇒ Object
738 739 740 |
# File 'lib/core_assertions.rb', line 738 def assert_warn(*args) assert_warning(*args) {$VERBOSE = false; yield} end |
#assert_warning(pat, msg = nil) ⇒ Object
726 727 728 729 730 731 732 733 734 735 736 |
# File 'lib/core_assertions.rb', line 726 def assert_warning(pat, msg = nil) result = nil stderr = EnvUtil.with_default_internal(of: pat) { EnvUtil.verbose_warning { result = yield } } msg = (msg) {diff pat, stderr} assert(pat === stderr, msg) result end |
#diff(exp, act) ⇒ Object
938 939 940 941 942 943 944 945 946 947 948 949 950 951 952 |
# File 'lib/core_assertions.rb', line 938 def diff(exp, act) require 'pp' q = PP.new(+"") q.guard_inspect_key do q.group(2, "expected: ") do q.pp exp end q.text q.newline q.group(2, "actual: ") do q.pp act end q.flush end q.output end |
#mu_pp(obj) ⇒ Object
93 94 95 |
# File 'lib/core_assertions.rb', line 93 def mu_pp(obj) #:nodoc: obj.pretty_inspect.chomp end |
#new_test_token ⇒ Object
954 955 956 957 |
# File 'lib/core_assertions.rb', line 954 def new_test_token token = "\e[7;1m#{$$.to_s}:#{Time.now.strftime('%s.%L')}:#{rand(0x10000).to_s(16)}:\e[m" return token.dump, Regexp.quote(token) end |
#prepare_syntax_check(code, fname = nil, mesg = nil, verbose: nil) ⇒ Object
251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 |
# File 'lib/core_assertions.rb', line 251 def prepare_syntax_check(code, fname = nil, mesg = nil, verbose: nil) fname ||= caller_locations(2, 1)[0] mesg ||= fname.to_s verbose, $VERBOSE = $VERBOSE, verbose case when Array === fname fname, line = *fname when defined?(fname.path) && defined?(fname.lineno) fname, line = fname.path, fname.lineno else line = 1 end yield(code, fname, line, (mesg) { if code.end_with?("\n") "```\n#{code}```\n" else "```\n#{code}\n```\n""no-newline" end }) ensure $VERBOSE = verbose end |
#separated_runner(token, out = nil) ⇒ Object
304 305 306 307 308 309 310 311 312 313 314 315 316 317 318 319 320 321 322 323 324 325 326 327 328 329 330 331 332 333 334 335 336 337 338 339 340 |
# File 'lib/core_assertions.rb', line 304 def separated_runner(token, out = nil) include(*Test::Unit::TestCase.ancestors.select {|c| !c.is_a?(Class) }) out = out ? IO.new(out, 'w') : STDOUT # avoid method redefinitions out_write = out.method(:write) integer_to_s = Integer.instance_method(:to_s) array_pack = Array.instance_method(:pack) marshal_dump = Marshal.method(:dump) assertions_ivar_set = Test::Unit::Assertions.method(:instance_variable_set) assertions_ivar_get = Test::Unit::Assertions.method(:instance_variable_get) Test::Unit::Assertions.module_eval do @_assertions = 0 undef _assertions= define_method(:_assertions=, ->(n) {assertions_ivar_set.call(:@_assertions, n)}) undef _assertions define_method(:_assertions, -> {assertions_ivar_get.call(:@_assertions)}) end # assume Method#call and UnboundMethod#bind_call need to work as the original at_exit { assertions = assertions_ivar_get.call(:@_assertions) out_write.call <<~OUT <error id="#{token}" assertions=#{integer_to_s.bind(assertions).call}> #{array_pack.bind([marshal_dump.call($!)]).call('m0')} </error id="#{token}"> OUT } if defined?(Test::Unit::Runner) Test::Unit::Runner.class_variable_set(:@@stop_auto_run, true) elsif defined?(Test::Unit::AutoRunner) Test::Unit::AutoRunner.need_auto_run = false end end |
#syntax_check(code, fname, line) ⇒ Object
149 150 151 152 153 154 155 |
# File 'lib/core_assertions.rb', line 149 def syntax_check(code, fname, line) code = code.dup.force_encoding(Encoding::UTF_8) RubyVM::InstructionSequence.compile(code, fname, fname, line) :ok ensure raise if SyntaxError === $! end |